import { Type } from "./type";
/**
 * Describes type of data columns to use for the data legend.
 */
export declare enum DataLegendSeriesValueType {
    /**
     * Specifies the Value data column for the category series and radial series
     */
    Value = 0,
    /**
     * Specifies the Low data column for the financial price series and range series
     */
    Low = 1,
    /**
     * Specifies the High data column for the financial price series and range series
     */
    High = 2,
    /**
     * Specifies the Open data column for the financial price series
     */
    Open = 3,
    /**
     * Specifies the Close data column for the financial price series
     */
    Close = 4,
    /**
     * Specifies the typical price column for the financial price series: (high + low + close) / 3
     */
    TypicalPrice = 5,
    /**
     * Specifies the average column between high/low for the range series
     */
    Average = 6,
    /**
     * Specifies the range column between high/low for the range series
     */
    Range = 7,
    /**
     * Specifies the Volume data column for the financial price series
     */
    Volume = 8,
    /**
     * Specifies the Change data column for the financial price series
     */
    Change = 9,
    /**
     * Specifies the Angle data column for the polar series
     */
    Angle = 10,
    /**
     * Specifies the Radius data column for the polar series, radial series, and bubble series
     */
    Radius = 11,
    /**
     * Specifies the Fill data column for the bubble series
     */
    Fill = 12,
    /**
     * Specifies the X data column for the bubble series and scatter series
     */
    XValue = 13,
    /**
     * Specifies the Y data column for the bubble series and scatter series
     */
    YValue = 14,
    /**
     * Specifies the summary data columns for all series
     */
    Summary = 15
}
/**
 * @hidden
 */
export declare let DataLegendSeriesValueType_$type: Type;
